[![GitHub release](https://img.shields.io/github/release/ViennaRNA/ViennaRNA.svg)](https://www.tbi.univie.ac.at/RNA/#download)
[![Build Status](https://travis-ci.org/ViennaRNA/ViennaRNA.svg?branch=master)](https://travis-ci.org/ViennaRNA/ViennaRNA)
[![Github All Releases](https://img.shields.io/github/downloads/ViennaRNA/ViennaRNA/total.svg)](https://github.com/ViennaRNA/ViennaRNA/releases)
[![Conda](https://img.shields.io/conda/v/bioconda/viennarna.svg)](https://anaconda.org/bioconda/viennarna)
[![AUR](https://img.shields.io/aur/version/viennarna.svg)](https://aur.archlinux.org/packages/viennarna/)

# ViennaRNA Package

## Version 2.4.9

See the [NEWS](NEWS) and [CHANGELOG.md](CHANGELOG.md) files for changes between versions.

## About

The ViennaRNA Package consists of a few stand alone programs and a 
library that you can link your own programs with. 

Together with this version we distribute the programs

- Kinfold,
- RNAforester,
- RNAlocmin, and
- Kinwalker

See the README files in the respective sub-directories.

The package allows you to

- predict minimum free energy secondary structures
- calculate the partition function for the ensemble of structures
- calculate suboptimal structures in a given energy range
- compute local structures in long sequences
- predict consensus secondary structures from a multiple sequence alignment
- predict melting curves
- search for sequences folding into a given structure
- compare two secondary structures 
- predict hybridization structures of two RNA molecules

The package includes Perl5, Python 2, and Python 3 modules that give
access to almost all functions of the C library from within the respective
scripting languages.

There is also a set of programs for analyzing sequence and distance
data using split decomposition, statistical geometry, and cluster methods.
They are not maintained any more and not built by default.

The code very rarely uses static arrays, and all programs should work for 
sequences up to a length of 32,700 (if you have huge amounts of memory that
is).

## Copyright

Please read the copyright notice in the file [COPYING](COPYING)!

If you're a commercial user and find these programs useful, please consider
supporting further developments with a donation.

## Availability

The most recent source code and documentation should always be available through
the [official ViennaRNA website](https://www.tbi.univie.ac.at/RNA) and through
[github](https://github.com/ViennaRNA/ViennaRNA).

## Compatibility

The package should be easily portable. It is known to compile without
modifications at least under
SunOS 5.x, IRIX 5.x and 6.x, Linux, and MS Windows with the Cygwin environment.
Other UNIX flavors should present no problems.
You need a compiler that understands ANSI C. See the [INSTALL](INSTALL) file for details.

## Executable Programs

The following executable programs are provided:
- `RNA2Dfold`       Compute MFE structure, partition function and representative sample structures of k,l neighborhoods
- `RNAaliduplex`    Predict conserved RNA-RNA interactions between two alignments
- `RNAalifold`      Calculate secondary structures for a set of aligned RNA sequences
- `RNAcofold`       Calculate secondary structures of two RNAs with dimerization
- `RNAdistance`     Calculate distances between RNA secondary structures
- `RNAduplex`       Compute the structure upon hybridization of two RNA strands
- `RNAeval`         Evaluate free energy of RNA sequences with given secondary structure
- `RNAfold`         Calculate minimum free energy secondary structures and partition function of RNAs
- `RNAheat`         Calculate the specific heat (melting curve) of an RNA sequence
- `RNAinverse`      Find RNA sequences with given secondary structure (sequence design)
- `RNALalifold`     Calculate locally stable secondary structures for a set of aligned RNAs
- `RNALfold`        Calculate locally stable secondary structures of long RNAs
- `RNApaln`         RNA alignment based on sequence base pairing propensities
- `RNApdist`        Calculate distances between thermodynamic RNA secondary structures ensembles
- `RNAparconv`      Convert energy parameter files from ViennaRNA 1.8 to 2.0 format
- `RNAPKplex`       Predict RNA secondary structures including pseudoknots
- `RNAplex`         Find targets of a query RNA
- `RNAplfold`       Calculate average pair probabilities for locally stable secondary structures
- `RNAplot`         Draw RNA Secondary Structures in PostScript, SVG, or GML
- `RNApvmin`        Calculate a perturbation vector that minimizes discripancies between predicted and observed pairing probabilities
- `RNAsnoop`        Find targets of a query H/ACA snoRNA
- `RNAsubopt`       Calculate suboptimal secondary structures of RNAs
- `RNAup`           Calculate the thermodynamics of RNA-RNA interactions 
- `AnalyseSeqs`     Analyse sequence data
- `AnalyseDists`    Analyse distance matrices

A couple of useful utilities can be found in the src/Utils directory.

All executables read from stdin and write to stdout and use command line
switches rather than menus to be easily usable in pipes.

For more detailed information see the man pages. Perl utilities contain
POD documentation that can be read by typing e.g.
```
perldoc relplot.pl
```

## References

If you use our software package, you may want to cite

[R. Lorenz et al. (2011)](https://almob.biomedcentral.com/articles/10.1186/1748-7188-6-26),
"ViennaRNA Package 2.0", Algorithms for Molecular Biology, 6:26

and

[I.L. Hofacker (1994)](https://link.springer.com/article/10.1007/BF00818163),
"Fast folding and comparison of RNA secondary structures",
Monatshefte fuer Chemie, Volume 125, Issue 2, pp 167-188


*Note, that the individual executable programs state their own list of references
in the corresponding man-pages.*

### Energy Parameters

Since version 2.0 the build-in energy parameters, also available as parameter
file [rna_turner2004.par](misc/rna_turner2004.par), are taken from:

[D.H. Mathews et al. (2004)](https://doi.org/10.1073/pnas.0401799101),
"Incorporating chemical modification constraints into a dynamic programming
algorithm for prediction of RNA secondary structure",
Proc. Natl. Acad. Sci. USA: 101, pp 7287-7292

and

[D.H. Turner et al. (2009)](https://dx.doi.org/10.1093/nar/gkp892), 
"NNDB: The nearest neighbor parameter database
for predicting stability of nucleic acid secondary structure",
Nucleic Acids Research: 38, pp 280-282.

For backward compatibility we also provide energy parameters from Turner et al.
1999 in the file [rna_turner1999.par](misc/rna_turner1999.par). Additionally,
a set of trained RNA energy parameters from Andronescou et al. 2007,
[rna_andronescou2007.par](misc/rna_andronescou2007.par), a set of RNA energy
parameters obtained by graft and grow genetic programming from Langdon
et al. 2018, [rna_langdon2018.par](misc/rna_langdon2018.par), and two sets
of DNA parameters, [dna_mathews1999.par](misc/dna_mathews1999.par) and
[dna_mathews2004.par](misc/dna_mathews2004.par), are included
in the package as well.

## Contact

We need your feedback! Send your comments, suggestions, and questions to
rna@tbi.univie.ac.at

Ivo Hofacker, Spring 2006
